@@ -10,6 +10,7 @@ import { homedir } from 'node:os';
 
 import type { MCPServerConfig } from '@google/gemini-cli-core';
 import {
+  debugLogger,
   GEMINI_DIR,
   getErrorMessage,
   type TelemetrySettings,
@@ -97,10 +98,10 @@ export function loadSettings(workspaceDir: string): Settings {
   }
 
   if (settingsErrors.length > 0) {
-    console.error('Errors loading settings:');
+    debugLogger.error('Errors loading settings:');
     for (const error of settingsErrors) {
-      console.error(`  Path: ${error.path}`);
-      console.error(`  Message: ${error.message}`);
+      debugLogger.error(`  Path: ${error.path}`);
+      debugLogger.error(`  Message: ${error.message}`);
     }
   }
 
