@@ -10,7 +10,10 @@ export function getGitHubToken(): string | undefined {
   return process.env['GITHUB_TOKEN'];
 }
 
-export async function fetchJson<T>(url: string): Promise<T> {
+export async function fetchJson<T>(
+  url: string,
+  redirectCount: number = 0,
+): Promise<T> {
   const headers: { 'User-Agent': string; Authorization?: string } = {
     'User-Agent': 'gemini-cli',
   };
@@ -21,6 +24,18 @@ export async function fetchJson<T>(url: string): Promise<T> {
   return new Promise((resolve, reject) => {
     https
       .get(url, { headers }, (res) => {
+        if (res.statusCode === 302 || res.statusCode === 301) {
+          if (redirectCount >= 10) {
+            return reject(new Error('Too many redirects'));
+          }
+          if (!res.headers.location) {
+            return reject(new Error('No location header in redirect response'));
+          }
+          fetchJson<T>(res.headers.location!, redirectCount++)
+            .then(resolve)
+            .catch(reject);
+          return;
+        }
         if (res.statusCode !== 200) {
           return reject(
             new Error(`Request failed with status code ${res.statusCode}`),
