@@ -7,6 +7,8 @@
 import { describe, it, expect } from 'vitest';
 import {
   getSettingsSchema,
+  SETTINGS_SCHEMA_DEFINITIONS,
+  type SettingCollectionDefinition,
   type SettingDefinition,
   type Settings,
   type SettingsSchema,
@@ -159,6 +161,10 @@ describe('SettingsSchema', () => {
       expect(
         getSettingsSchema().ui.properties.showMemoryUsage.showInDialog,
       ).toBe(true);
+      expect(
+        getSettingsSchema().ui.properties.footer.properties
+          .hideContextPercentage.showInDialog,
+      ).toBe(true);
       expect(getSettingsSchema().general.properties.vimMode.showInDialog).toBe(
         true,
       );
@@ -316,19 +322,89 @@ describe('SettingsSchema', () => {
       ).toBe('Enable debug logging of keystrokes to the console.');
     });
 
-    it('should have useModelRouter setting in schema', () => {
+    it('should have previewFeatures setting in schema', () => {
       expect(
-        getSettingsSchema().experimental.properties.useModelRouter,
+        getSettingsSchema().general.properties.previewFeatures,
       ).toBeDefined();
+      expect(getSettingsSchema().general.properties.previewFeatures.type).toBe(
+        'boolean',
+      );
       expect(
-        getSettingsSchema().experimental.properties.useModelRouter.type,
-      ).toBe('boolean');
+        getSettingsSchema().general.properties.previewFeatures.category,
+      ).toBe('General');
       expect(
-        getSettingsSchema().experimental.properties.useModelRouter.category,
-      ).toBe('Experimental');
+        getSettingsSchema().general.properties.previewFeatures.default,
+      ).toBe(false);
       expect(
-        getSettingsSchema().experimental.properties.useModelRouter.default,
+        getSettingsSchema().general.properties.previewFeatures.requiresRestart,
+      ).toBe(false);
+      expect(
+        getSettingsSchema().general.properties.previewFeatures.showInDialog,
       ).toBe(true);
+      expect(
+        getSettingsSchema().general.properties.previewFeatures.description,
+      ).toBe('Enable preview features (e.g., preview models).');
+    });
+
+    it('should have isModelAvailabilityServiceEnabled setting in schema', () => {
+      const setting =
+        getSettingsSchema().experimental.properties
+          .isModelAvailabilityServiceEnabled;
+      expect(setting).toBeDefined();
+      expect(setting.type).toBe('boolean');
+      expect(setting.category).toBe('Experimental');
+      expect(setting.default).toBe(false);
+      expect(setting.requiresRestart).toBe(true);
+      expect(setting.showInDialog).toBe(false);
+      expect(setting.description).toBe(
+        'Enable model routing using new availability service.',
+      );
+    });
+  });
+
+  it('has JSON schema definitions for every referenced ref', () => {
+    const schema = getSettingsSchema();
+    const referenced = new Set<string>();
+
+    const visitDefinition = (definition: SettingDefinition) => {
+      if (definition.ref) {
+        referenced.add(definition.ref);
+        expect(SETTINGS_SCHEMA_DEFINITIONS).toHaveProperty(definition.ref);
+      }
+      if (definition.properties) {
+        Object.values(definition.properties).forEach(visitDefinition);
+      }
+      if (definition.items) {
+        visitCollection(definition.items);
+      }
+      if (definition.additionalProperties) {
+        visitCollection(definition.additionalProperties);
+      }
+    };
+
+    const visitCollection = (collection: SettingCollectionDefinition) => {
+      if (collection.ref) {
+        referenced.add(collection.ref);
+        expect(SETTINGS_SCHEMA_DEFINITIONS).toHaveProperty(collection.ref);
+        return;
+      }
+      if (collection.properties) {
+        Object.values(collection.properties).forEach(visitDefinition);
+      }
+      if (collection.type === 'array' && collection.properties) {
+        Object.values(collection.properties).forEach(visitDefinition);
+      }
+    };
+
+    Object.values(schema).forEach(visitDefinition);
+
+    // Ensure definitions map doesn't accumulate stale entries.
+    Object.keys(SETTINGS_SCHEMA_DEFINITIONS).forEach((key) => {
+      if (!referenced.has(key)) {
+        throw new Error(
+          `Definition "${key}" is exported but never referenced in the schema`,
+        );
+      }
     });
   });
 });
