@@ -7,7 +7,9 @@
 import type {
   Config,
   ToolCallRequestInfo,
+  ResumedSessionData,
   CompletedToolCall,
+  UserFeedbackPayload,
 } from '@google/gemini-cli-core';
 import { isSlashCommand } from './ui/utils/commandUtils.js';
 import type { LoadedSettings } from './config/settings.js';
@@ -24,10 +26,15 @@ import {
   JsonStreamEventType,
   uiTelemetryService,
   debugLogger,
+  coreEvents,
+  CoreEvent,
+  createWorkingStdio,
 } from '@google/gemini-cli-core';
 
 import type { Content, Part } from '@google/genai';
+import readline from 'node:readline';
 
+import { convertSessionToHistoryFormats } from './ui/hooks/useSessionBrowser.js';
 import { handleSlashCommand } from './nonInteractiveCliCommands.js';
 import { ConsolePatcher } from './ui/utils/ConsolePatcher.js';
 import { handleAtCommand } from './ui/hooks/atCommandProcessor.js';
@@ -37,18 +44,47 @@ import {
   handleCancellationError,
   handleMaxTurnsExceededError,
 } from './utils/errors.js';
+import { TextOutput } from './ui/utils/textOutput.js';
 
-export async function runNonInteractive(
-  config: Config,
-  settings: LoadedSettings,
-  input: string,
-  prompt_id: string,
-): Promise<void> {
+interface RunNonInteractiveParams {
+  config: Config;
+  settings: LoadedSettings;
+  input: string;
+  prompt_id: string;
+  hasDeprecatedPromptArg?: boolean;
+  resumedSessionData?: ResumedSessionData;
+}
+
+export async function runNonInteractive({
+  config,
+  settings,
+  input,
+  prompt_id,
+  hasDeprecatedPromptArg,
+  resumedSessionData,
+}: RunNonInteractiveParams): Promise<void> {
   return promptIdContext.run(prompt_id, async () => {
     const consolePatcher = new ConsolePatcher({
       stderr: true,
       debugMode: config.getDebugMode(),
+      onNewMessage: (msg) => {
+        coreEvents.emitConsoleLog(msg.type, msg.content);
+      },
     });
+    const { stdout: workingStdout } = createWorkingStdio();
+    const textOutput = new TextOutput(workingStdout);
+
+    const handleUserFeedback = (payload: UserFeedbackPayload) => {
+      const prefix = payload.severity.toUpperCase();
+      process.stderr.write(`[${prefix}] ${payload.message}\n`);
+      if (payload.error && config.getDebugMode()) {
+        const errorToLog =
+          payload.error instanceof Error
+            ? payload.error.stack || payload.error.message
+            : String(payload.error);
+        process.stderr.write(`${errorToLog}\n`);
+      }
+    };
 
     const startTime = Date.now();
     const streamFormatter =
@@ -56,8 +92,98 @@ export async function runNonInteractive(
         ? new StreamJsonFormatter()
         : null;
 
+    const abortController = new AbortController();
+
+    // Track cancellation state
+    let isAborting = false;
+    let cancelMessageTimer: NodeJS.Timeout | null = null;
+
+    // Setup stdin listener for Ctrl+C detection
+    let stdinWasRaw = false;
+    let rl: readline.Interface | null = null;
+
+    const setupStdinCancellation = () => {
+      // Only setup if stdin is a TTY (user can interact)
+      if (!process.stdin.isTTY) {
+        return;
+      }
+
+      // Save original raw mode state
+      stdinWasRaw = process.stdin.isRaw || false;
+
+      // Enable raw mode to capture individual keypresses
+      process.stdin.setRawMode(true);
+      process.stdin.resume();
+
+      // Setup readline to emit keypress events
+      rl = readline.createInterface({
+        input: process.stdin,
+        escapeCodeTimeout: 0,
+      });
+      readline.emitKeypressEvents(process.stdin, rl);
+
+      // Listen for Ctrl+C
+      const keypressHandler = (
+        str: string,
+        key: { name?: string; ctrl?: boolean },
+      ) => {
+        // Detect Ctrl+C: either ctrl+c key combo or raw character code 3
+        if ((key && key.ctrl && key.name === 'c') || str === '\u0003') {
+          // Only handle once
+          if (isAborting) {
+            return;
+          }
+
+          isAborting = true;
+
+          // Only show message if cancellation takes longer than 200ms
+          // This reduces verbosity for fast cancellations
+          cancelMessageTimer = setTimeout(() => {
+            process.stderr.write('\nCancelling...\n');
+          }, 200);
+
+          abortController.abort();
+          // Note: Don't exit here - let the abort flow through the system
+          // and trigger handleCancellationError() which will exit with proper code
+        }
+      };
+
+      process.stdin.on('keypress', keypressHandler);
+    };
+
+    const cleanupStdinCancellation = () => {
+      // Clear any pending cancel message timer
+      if (cancelMessageTimer) {
+        clearTimeout(cancelMessageTimer);
+        cancelMessageTimer = null;
+      }
+
+      // Cleanup readline and stdin listeners
+      if (rl) {
+        rl.close();
+        rl = null;
+      }
+
+      // Remove keypress listener
+      process.stdin.removeAllListeners('keypress');
+
+      // Restore stdin to original state
+      if (process.stdin.isTTY) {
+        process.stdin.setRawMode(stdinWasRaw);
+        process.stdin.pause();
+      }
+    };
+
+    let errorToHandle: unknown | undefined;
     try {
       consolePatcher.patch();
+
+      // Setup stdin cancellation listener
+      setupStdinCancellation();
+
+      coreEvents.on(CoreEvent.UserFeedback, handleUserFeedback);
+      coreEvents.drainBacklogs();
+
       // Handle EPIPE errors when the output is piped to a command that closes early.
       process.stdout.on('error', (err: NodeJS.ErrnoException) => {
         if (err.code === 'EPIPE') {
@@ -68,6 +194,16 @@ export async function runNonInteractive(
 
       const geminiClient = config.getGeminiClient();
 
+      // Initialize chat.  Resume if resume data is passed.
+      if (resumedSessionData) {
+        await geminiClient.resumeChat(
+          convertSessionToHistoryFormats(
+            resumedSessionData.conversation.messages,
+          ).clientHistory,
+          resumedSessionData,
+        );
+      }
+
       // Emit init event for streaming JSON
       if (streamFormatter) {
         streamFormatter.emitEvent({
@@ -78,8 +214,6 @@ export async function runNonInteractive(
         });
       }
 
-      const abortController = new AbortController();
-
       let query: Part[] | undefined;
 
       if (isSlashCommand(input)) {
@@ -130,6 +264,21 @@ export async function runNonInteractive(
       let currentMessages: Content[] = [{ role: 'user', parts: query }];
 
       let turnCount = 0;
+      const deprecateText =
+        'The --prompt (-p) flag has been deprecated and will be removed in a future version. Please use a positional argument for your prompt. See gemini --help for more information.\n';
+      if (hasDeprecatedPromptArg) {
+        if (streamFormatter) {
+          streamFormatter.emitEvent({
+            type: JsonStreamEventType.MESSAGE,
+            timestamp: new Date().toISOString(),
+            role: 'assistant',
+            content: deprecateText,
+            delta: true,
+          });
+        } else {
+          process.stderr.write(deprecateText);
+        }
+      }
       while (true) {
         turnCount++;
         if (
@@ -164,7 +313,9 @@ export async function runNonInteractive(
             } else if (config.getOutputFormat() === OutputFormat.JSON) {
               responseText += event.value;
             } else {
-              process.stdout.write(event.value);
+              if (event.value) {
+                textOutput.write(event.value);
+              }
             }
           } else if (event.type === GeminiEventType.ToolCallRequest) {
             if (streamFormatter) {
@@ -195,10 +346,13 @@ export async function runNonInteractive(
                 message: 'Maximum session turns exceeded',
               });
             }
+          } else if (event.type === GeminiEventType.Error) {
+            throw event.value.error;
           }
         }
 
         if (toolCallRequests.length > 0) {
+          textOutput.ensureTrailingNewline();
           const toolResponseParts: Part[] = [];
           const completedToolCalls: CompletedToolCall[] = [];
 
@@ -276,20 +430,28 @@ export async function runNonInteractive(
           } else if (config.getOutputFormat() === OutputFormat.JSON) {
             const formatter = new JsonFormatter();
             const stats = uiTelemetryService.getMetrics();
-            process.stdout.write(formatter.format(responseText, stats));
+            textOutput.write(formatter.format(responseText, stats));
           } else {
-            process.stdout.write('\n'); // Ensure a final newline
+            textOutput.ensureTrailingNewline(); // Ensure a final newline
           }
           return;
         }
       }
     } catch (error) {
-      handleError(error, config);
+      errorToHandle = error;
     } finally {
+      // Cleanup stdin cancellation before other cleanup
+      cleanupStdinCancellation();
+
       consolePatcher.cleanup();
+      coreEvents.off(CoreEvent.UserFeedback, handleUserFeedback);
       if (isTelemetrySdkInitialized()) {
         await shutdownTelemetry(config);
       }
     }
+
+    if (errorToHandle) {
+      handleError(errorToHandle, config);
+    }
   });
 }
