@@ -66,7 +66,7 @@ vi.mock('../ui/commands/corgiCommand.js', () => ({ corgiCommand: {} }));
 vi.mock('../ui/commands/docsCommand.js', () => ({ docsCommand: {} }));
 vi.mock('../ui/commands/editorCommand.js', () => ({ editorCommand: {} }));
 vi.mock('../ui/commands/extensionsCommand.js', () => ({
-  extensionsCommand: {},
+  extensionsCommand: () => ({}),
 }));
 vi.mock('../ui/commands/helpCommand.js', () => ({ helpCommand: {} }));
 vi.mock('../ui/commands/memoryCommand.js', () => ({ memoryCommand: {} }));
@@ -75,6 +75,7 @@ vi.mock('../ui/commands/modelCommand.js', () => ({
 }));
 vi.mock('../ui/commands/privacyCommand.js', () => ({ privacyCommand: {} }));
 vi.mock('../ui/commands/quitCommand.js', () => ({ quitCommand: {} }));
+vi.mock('../ui/commands/resumeCommand.js', () => ({ resumeCommand: {} }));
 vi.mock('../ui/commands/statsCommand.js', () => ({ statsCommand: {} }));
 vi.mock('../ui/commands/themeCommand.js', () => ({ themeCommand: {} }));
 vi.mock('../ui/commands/toolsCommand.js', () => ({ toolsCommand: {} }));
@@ -95,7 +96,8 @@ describe('BuiltinCommandLoader', () => {
     vi.clearAllMocks();
     mockConfig = {
       getFolderTrust: vi.fn().mockReturnValue(true),
-      getUseModelRouter: () => false,
+      getEnableMessageBusIntegration: () => false,
+      getEnableExtensionReloading: () => false,
     } as unknown as Config;
 
     restoreCommandMock.mockReturnValue({
@@ -166,26 +168,26 @@ describe('BuiltinCommandLoader', () => {
     expect(permissionsCmd).toBeUndefined();
   });
 
-  it('should include modelCommand when getUseModelRouter is true', async () => {
-    const mockConfigWithModelRouter = {
+  it('should include policies command when message bus integration is enabled', async () => {
+    const mockConfigWithMessageBus = {
       ...mockConfig,
-      getUseModelRouter: () => true,
+      getEnableMessageBusIntegration: () => true,
     } as unknown as Config;
-    const loader = new BuiltinCommandLoader(mockConfigWithModelRouter);
+    const loader = new BuiltinCommandLoader(mockConfigWithMessageBus);
     const commands = await loader.loadCommands(new AbortController().signal);
-    const modelCmd = commands.find((c) => c.name === 'model');
-    expect(modelCmd).toBeDefined();
+    const policiesCmd = commands.find((c) => c.name === 'policies');
+    expect(policiesCmd).toBeDefined();
   });
 
-  it('should not include modelCommand when getUseModelRouter is false', async () => {
-    const mockConfigWithoutModelRouter = {
+  it('should exclude policies command when message bus integration is disabled', async () => {
+    const mockConfigWithoutMessageBus = {
       ...mockConfig,
-      getUseModelRouter: () => false,
+      getEnableMessageBusIntegration: () => false,
     } as unknown as Config;
-    const loader = new BuiltinCommandLoader(mockConfigWithoutModelRouter);
+    const loader = new BuiltinCommandLoader(mockConfigWithoutMessageBus);
     const commands = await loader.loadCommands(new AbortController().signal);
-    const modelCmd = commands.find((c) => c.name === 'model');
-    expect(modelCmd).toBeUndefined();
+    const policiesCmd = commands.find((c) => c.name === 'policies');
+    expect(policiesCmd).toBeUndefined();
   });
 });
 
@@ -196,8 +198,9 @@ describe('BuiltinCommandLoader profile', () => {
     vi.resetModules();
     mockConfig = {
       getFolderTrust: vi.fn().mockReturnValue(false),
-      getUseModelRouter: () => false,
       getCheckpointingEnabled: () => false,
+      getEnableMessageBusIntegration: () => false,
+      getEnableExtensionReloading: () => false,
     } as unknown as Config;
   });
 
