@@ -5,22 +5,25 @@
  */
 
 import type React from 'react';
-import { useCallback } from 'react';
+import { useCallback, useState } from 'react';
 import { Box, Text } from 'ink';
 import { theme } from '../semantic-colors.js';
 import { RadioButtonSelect } from '../components/shared/RadioButtonSelect.js';
-import type { LoadedSettings } from '../../config/settings.js';
+import type {
+  LoadableSettingScope,
+  LoadedSettings,
+} from '../../config/settings.js';
 import { SettingScope } from '../../config/settings.js';
 import {
   AuthType,
   clearCachedCredentialFile,
-  debugLogger,
   type Config,
 } from '@google/gemini-cli-core';
 import { useKeypress } from '../hooks/useKeypress.js';
 import { AuthState } from '../types.js';
 import { runExitCleanup } from '../../utils/cleanup.js';
 import { validateAuthMethodWithSettings } from './useAuth.js';
+import { RELAUNCH_EXIT_CODE } from '../../utils/processUtils.js';
 
 interface AuthDialogProps {
   config: Config;
@@ -37,6 +40,7 @@ export function AuthDialog({
   authError,
   onAuthError,
 }: AuthDialogProps): React.JSX.Element {
+  const [exiting, setExiting] = useState(false);
   let items = [
     {
       label: 'Login with Google',
@@ -47,11 +51,19 @@ export function AuthDialog({
       ? [
           {
             label: 'Use Cloud Shell user credentials',
-            value: AuthType.CLOUD_SHELL,
-            key: AuthType.CLOUD_SHELL,
+            value: AuthType.COMPUTE_ADC,
+            key: AuthType.COMPUTE_ADC,
           },
         ]
-      : []),
+      : process.env['GEMINI_CLI_USE_COMPUTE_ADC'] === 'true'
+        ? [
+            {
+              label: 'Use metadata server application default credentials',
+              value: AuthType.COMPUTE_ADC,
+              key: AuthType.COMPUTE_ADC,
+            },
+          ]
+        : []),
     {
       label: 'Use Gemini API Key',
       value: AuthType.USE_GEMINI,
@@ -99,8 +111,14 @@ export function AuthDialog({
   }
 
   const onSelect = useCallback(
-    async (authType: AuthType | undefined, scope: SettingScope) => {
+    async (authType: AuthType | undefined, scope: LoadableSettingScope) => {
+      if (exiting) {
+        return;
+      }
       if (authType) {
+        const isInitialAuthSelection =
+          !settings.merged.security?.auth?.selectedType;
+
         await clearCachedCredentialFile();
 
         settings.setValue(scope, 'security.auth.selectedType', authType);
@@ -108,20 +126,27 @@ export function AuthDialog({
           authType === AuthType.LOGIN_WITH_GOOGLE &&
           config.isBrowserLaunchSuppressed()
         ) {
-          runExitCleanup();
-          debugLogger.log(
-            `
-----------------------------------------------------------------
-Logging in with Google... Please restart Gemini CLI to continue.
-----------------------------------------------------------------
-            `,
-          );
-          process.exit(0);
+          setExiting(true);
+          setTimeout(async () => {
+            await runExitCleanup();
+            process.exit(RELAUNCH_EXIT_CODE);
+          }, 100);
+          return;
+        }
+
+        if (authType === AuthType.USE_GEMINI) {
+          if (isInitialAuthSelection && process.env['GEMINI_API_KEY']) {
+            setAuthState(AuthState.Unauthenticated);
+            return;
+          } else {
+            setAuthState(AuthState.AwaitingApiKeyInput);
+            return;
+          }
         }
       }
       setAuthState(AuthState.Unauthenticated);
     },
-    [settings, config, setAuthState],
+    [settings, config, setAuthState, exiting],
   );
 
   const handleAuthSelect = (authMethod: AuthType) => {
@@ -154,53 +179,72 @@ Logging in with Google... Please restart Gemini CLI to continue.
     { isActive: true },
   );
 
+  if (exiting) {
+    return (
+      <Box
+        borderStyle="round"
+        borderColor={theme.border.focused}
+        flexDirection="row"
+        padding={1}
+        width="100%"
+        alignItems="flex-start"
+      >
+        <Text color={theme.text.primary}>
+          Logging in with Google... Restarting Gemini CLI to continue.
+        </Text>
+      </Box>
+    );
+  }
+
   return (
     <Box
       borderStyle="round"
-      borderColor={theme.border.default}
-      flexDirection="column"
+      borderColor={theme.border.focused}
+      flexDirection="row"
       padding={1}
       width="100%"
+      alignItems="flex-start"
     >
-      <Text bold color={theme.text.primary}>
-        Get started
-      </Text>
-      <Box marginTop={1}>
-        <Text color={theme.text.primary}>
-          How would you like to authenticate for this project?
+      <Text color={theme.text.accent}>? </Text>
+      <Box flexDirection="column" flexGrow={1}>
+        <Text bold color={theme.text.primary}>
+          Get started
         </Text>
-      </Box>
-      <Box marginTop={1}>
-        <RadioButtonSelect
-          items={items}
-          initialIndex={initialAuthIndex}
-          onSelect={handleAuthSelect}
-          onHighlight={() => {
-            onAuthError(null);
-          }}
-        />
-      </Box>
-      {authError && (
         <Box marginTop={1}>
-          <Text color={theme.status.error}>{authError}</Text>
+          <Text color={theme.text.primary}>
+            How would you like to authenticate for this project?
+          </Text>
+        </Box>
+        <Box marginTop={1}>
+          <RadioButtonSelect
+            items={items}
+            initialIndex={initialAuthIndex}
+            onSelect={handleAuthSelect}
+            onHighlight={() => {
+              onAuthError(null);
+            }}
+          />
+        </Box>
+        {authError && (
+          <Box marginTop={1}>
+            <Text color={theme.status.error}>{authError}</Text>
+          </Box>
+        )}
+        <Box marginTop={1}>
+          <Text color={theme.text.secondary}>(Use Enter to select)</Text>
+        </Box>
+        <Box marginTop={1}>
+          <Text color={theme.text.primary}>
+            Terms of Services and Privacy Notice for Gemini CLI
+          </Text>
+        </Box>
+        <Box marginTop={1}>
+          <Text color={theme.text.link}>
+            {
+              'https://github.com/google-gemini/gemini-cli/blob/main/docs/tos-privacy.md'
+            }
+          </Text>
         </Box>
-      )}
-      <Box marginTop={1}>
-        <Text color={theme.text.secondary}>
-          (Use Enter to select, Esc to close)
-        </Text>
-      </Box>
-      <Box marginTop={1}>
-        <Text color={theme.text.primary}>
-          Terms of Services and Privacy Notice for Gemini CLI
-        </Text>
-      </Box>
-      <Box marginTop={1}>
-        <Text color={theme.text.link}>
-          {
-            'https://github.com/google-gemini/gemini-cli/blob/main/docs/tos-privacy.md'
-          }
-        </Text>
       </Box>
     </Box>
   );
