@@ -28,6 +28,7 @@ const authCommand: SlashCommand = {
   name: 'auth',
   description: 'Authenticate with an OAuth-enabled MCP server',
   kind: CommandKind.BUILT_IN,
+  autoExecute: false,
   action: async (
     context: CommandContext,
     args: string,
@@ -43,7 +44,7 @@ const authCommand: SlashCommand = {
       };
     }
 
-    const mcpServers = config.getMcpServers() || {};
+    const mcpServers = config.getMcpClientManager()?.getMcpServers() ?? {};
 
     if (!serverName) {
       // List servers that support OAuth
@@ -119,20 +120,20 @@ const authCommand: SlashCommand = {
       );
 
       // Trigger tool re-discovery to pick up authenticated server
-      const toolRegistry = config.getToolRegistry();
-      if (toolRegistry) {
+      const mcpClientManager = config.getMcpClientManager();
+      if (mcpClientManager) {
         context.ui.addItem(
           {
             type: 'info',
-            text: `Re-discovering tools from '${serverName}'...`,
+            text: `Restarting MCP server '${serverName}'...`,
           },
           Date.now(),
         );
-        await toolRegistry.discoverToolsForServer(serverName);
+        await mcpClientManager.restartServer(serverName);
       }
       // Update the client with the new tools
       const geminiClient = config.getGeminiClient();
-      if (geminiClient) {
+      if (geminiClient?.isInitialized()) {
         await geminiClient.setTools();
       }
 
@@ -158,7 +159,7 @@ const authCommand: SlashCommand = {
     const { config } = context.services;
     if (!config) return [];
 
-    const mcpServers = config.getMcpServers() || {};
+    const mcpServers = config.getMcpClientManager()?.getMcpServers() || {};
     return Object.keys(mcpServers).filter((name) =>
       name.startsWith(partialArg),
     );
@@ -188,9 +189,10 @@ const listAction = async (
     };
   }
 
-  const mcpServers = config.getMcpServers() || {};
+  const mcpServers = config.getMcpClientManager()?.getMcpServers() || {};
   const serverNames = Object.keys(mcpServers);
-  const blockedMcpServers = config.getBlockedMcpServers() || [];
+  const blockedMcpServers =
+    config.getMcpClientManager()?.getBlockedMcpServers() || [];
 
   const connectingServers = serverNames.filter(
     (name) => getMCPServerStatus(name) === MCPServerStatus.CONNECTING,
@@ -264,6 +266,7 @@ const listCommand: SlashCommand = {
   altNames: ['ls', 'nodesc', 'nodescription'],
   description: 'List configured MCP servers and tools',
   kind: CommandKind.BUILT_IN,
+  autoExecute: true,
   action: (context) => listAction(context),
 };
 
@@ -272,6 +275,7 @@ const descCommand: SlashCommand = {
   altNames: ['description'],
   description: 'List configured MCP servers and tools with descriptions',
   kind: CommandKind.BUILT_IN,
+  autoExecute: true,
   action: (context) => listAction(context, true),
 };
 
@@ -280,6 +284,7 @@ const schemaCommand: SlashCommand = {
   description:
     'List configured MCP servers and tools with descriptions and schemas',
   kind: CommandKind.BUILT_IN,
+  autoExecute: true,
   action: (context) => listAction(context, true, true),
 };
 
@@ -287,6 +292,7 @@ const refreshCommand: SlashCommand = {
   name: 'refresh',
   description: 'Restarts MCP servers',
   kind: CommandKind.BUILT_IN,
+  autoExecute: true,
   action: async (
     context: CommandContext,
   ): Promise<void | SlashCommandActionReturn> => {
@@ -299,12 +305,12 @@ const refreshCommand: SlashCommand = {
       };
     }
 
-    const toolRegistry = config.getToolRegistry();
-    if (!toolRegistry) {
+    const mcpClientManager = config.getMcpClientManager();
+    if (!mcpClientManager) {
       return {
         type: 'message',
         messageType: 'error',
-        content: 'Could not retrieve tool registry.',
+        content: 'Could not retrieve mcp client manager.',
       };
     }
 
@@ -316,11 +322,11 @@ const refreshCommand: SlashCommand = {
       Date.now(),
     );
 
-    await toolRegistry.restartMcpServers();
+    await mcpClientManager.restart();
 
     // Update the client with the new tools
     const geminiClient = config.getGeminiClient();
-    if (geminiClient) {
+    if (geminiClient?.isInitialized()) {
       await geminiClient.setTools();
     }
 
@@ -335,6 +341,7 @@ export const mcpCommand: SlashCommand = {
   name: 'mcp',
   description: 'Manage configured Model Context Protocol (MCP) servers',
   kind: CommandKind.BUILT_IN,
+  autoExecute: false,
   subCommands: [
     listCommand,
     descCommand,
