@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import { render } from 'ink-testing-library';
+import { render } from '../../../test-utils/render.js';
 import { describe, it, expect, vi } from 'vitest';
 import { McpStatus } from './McpStatus.js';
 import { MCPServerStatus } from '@google/gemini-cli-core';
@@ -46,32 +46,36 @@ describe('McpStatus', () => {
   };
 
   it('renders correctly with a connected server', () => {
-    const { lastFrame } = render(<McpStatus {...baseProps} />);
+    const { lastFrame, unmount } = render(<McpStatus {...baseProps} />);
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly with authenticated OAuth status', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <McpStatus {...baseProps} authStatus={{ 'server-1': 'authenticated' }} />,
     );
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly with expired OAuth status', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <McpStatus {...baseProps} authStatus={{ 'server-1': 'expired' }} />,
     );
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly with unauthenticated OAuth status', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <McpStatus
         {...baseProps}
         authStatus={{ 'server-1': 'unauthenticated' }}
       />,
     );
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly with a disconnected server', async () => {
@@ -79,26 +83,29 @@ describe('McpStatus', () => {
       await import('@google/gemini-cli-core'),
       'getMCPServerStatus',
     ).mockReturnValue(MCPServerStatus.DISCONNECTED);
-    const { lastFrame } = render(<McpStatus {...baseProps} />);
+    const { lastFrame, unmount } = render(<McpStatus {...baseProps} />);
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly when discovery is in progress', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <McpStatus {...baseProps} discoveryInProgress={true} />,
     );
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly with schema enabled', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <McpStatus {...baseProps} showSchema={true} />,
     );
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly with parametersJsonSchema', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <McpStatus
         {...baseProps}
         tools={[
@@ -120,10 +127,11 @@ describe('McpStatus', () => {
       />,
     );
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly with prompts', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <McpStatus
         {...baseProps}
         prompts={[
@@ -136,22 +144,25 @@ describe('McpStatus', () => {
       />,
     );
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly with a blocked server', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <McpStatus
         {...baseProps}
         blockedServers={[{ name: 'server-1', extensionName: 'test-extension' }]}
       />,
     );
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 
   it('renders correctly with a connecting server', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <McpStatus {...baseProps} connectingServers={['server-1']} />,
     );
     expect(lastFrame()).toMatchSnapshot();
+    unmount();
   });
 });
