@@ -9,12 +9,29 @@ import { join } from 'node:path';
 import { Storage } from '@google/gemini-cli-core';
 
 const cleanupFunctions: Array<(() => void) | (() => Promise<void>)> = [];
+const syncCleanupFunctions: Array<() => void> = [];
 
 export function registerCleanup(fn: (() => void) | (() => Promise<void>)) {
   cleanupFunctions.push(fn);
 }
 
+export function registerSyncCleanup(fn: () => void) {
+  syncCleanupFunctions.push(fn);
+}
+
+export function runSyncCleanup() {
+  for (const fn of syncCleanupFunctions) {
+    try {
+      fn();
+    } catch (_) {
+      // Ignore errors during cleanup.
+    }
+  }
+  syncCleanupFunctions.length = 0;
+}
+
 export async function runExitCleanup() {
+  runSyncCleanup();
   for (const fn of cleanupFunctions) {
     try {
       await fn();
