@@ -245,4 +245,15 @@ describe('GitService', () => {
       expect(hoistedMockCommit).not.toHaveBeenCalled();
     });
   });
+
+  describe('createFileSnapshot', () => {
+    it('should commit with --no-verify flag', async () => {
+      const service = new GitService(projectRoot, storage);
+      await service.initialize();
+      await service.createFileSnapshot('test commit');
+      expect(hoistedMockCommit).toHaveBeenCalledWith('test commit', {
+        '--no-verify': null,
+      });
+    });
+  });
 });
