@@ -42,6 +42,7 @@ interface CustomMatchers<R = unknown> {
   toHaveMetadataValue: ([key, value]: [EventMetadataKey, string]) => R;
   toHaveEventName: (name: EventNames) => R;
   toHaveMetadataKey: (key: EventMetadataKey) => R;
+  toHaveGwsExperiments: (exps: number[]) => R;
 }
 
 declare module 'vitest' {
@@ -65,7 +66,6 @@ expect.extend({
     received: LogEventEntry[],
     [key, value]: [EventMetadataKey, string],
   ) {
-    const { isNot } = this;
     const event = JSON.parse(received[0].source_extension_json) as LogEvent;
     const metadata = event['event_metadata'][0];
     const data = metadata.find((m) => m.gemini_cli_key === key)?.value;
@@ -74,8 +74,7 @@ expect.extend({
 
     return {
       pass,
-      message: () =>
-        `event ${received} does${isNot ? ' not' : ''} have ${value}}`,
+      message: () => `event ${received} should have: ${value}. Found: ${data}`,
     };
   },
 
@@ -133,7 +132,11 @@ describe('ClearcutLogger', () => {
   });
 
   function setup({
-    config = {} as Partial<ConfigParameters>,
+    config = {
+      experiments: {
+        experimentIds: [123, 456, 789],
+      },
+    } as unknown as Partial<ConfigParameters>,
     lifetimeGoogleAccounts = 1,
     cachedGoogleAccount = 'test@google.com',
   } = {}) {
@@ -211,45 +214,6 @@ describe('ClearcutLogger', () => {
       });
     });
 
-    it('logs the current surface from a github action', () => {
-      const { logger } = setup({});
-
-      vi.stubEnv('GITHUB_SHA', '8675309');
-
-      const event = logger?.createLogEvent(EventNames.CHAT_COMPRESSION, []);
-
-      expect(event?.event_metadata[0]).toContainEqual({
-        gemini_cli_key: EventMetadataKey.GEMINI_CLI_SURFACE,
-        value: 'GitHub',
-      });
-    });
-
-    it('logs the current surface from Cloud Shell via EDITOR_IN_CLOUD_SHELL', () => {
-      const { logger } = setup({});
-
-      vi.stubEnv('EDITOR_IN_CLOUD_SHELL', 'true');
-
-      const event = logger?.createLogEvent(EventNames.CHAT_COMPRESSION, []);
-
-      expect(event?.event_metadata[0]).toContainEqual({
-        gemini_cli_key: EventMetadataKey.GEMINI_CLI_SURFACE,
-        value: 'cloudshell',
-      });
-    });
-
-    it('logs the current surface from Cloud Shell via CLOUD_SHELL', () => {
-      const { logger } = setup({});
-
-      vi.stubEnv('CLOUD_SHELL', 'true');
-
-      const event = logger?.createLogEvent(EventNames.CHAT_COMPRESSION, []);
-
-      expect(event?.event_metadata[0]).toContainEqual({
-        gemini_cli_key: EventMetadataKey.GEMINI_CLI_SURFACE,
-        value: 'cloudshell',
-      });
-    });
-
     it('logs default metadata', () => {
       // Define expected values
       const session_id = 'my-session-id';
@@ -329,23 +293,9 @@ describe('ClearcutLogger', () => {
       });
     });
 
-    it('logs the current surface', () => {
-      const { logger } = setup({});
-
-      vi.stubEnv('TERM_PROGRAM', 'vscode');
-      vi.stubEnv('SURFACE', 'ide-1234');
-
-      const event = logger?.createLogEvent(EventNames.API_ERROR, []);
-
-      expect(event?.event_metadata[0]).toContainEqual({
-        gemini_cli_key: EventMetadataKey.GEMINI_CLI_SURFACE,
-        value: 'ide-1234',
-      });
-    });
-
     it('logs all user settings', () => {
       const { logger } = setup({
-        config: { useSmartEdit: true, useModelRouter: true },
+        config: { useSmartEdit: true },
       });
 
       vi.stubEnv('TERM_PROGRAM', 'vscode');
@@ -359,68 +309,124 @@ describe('ClearcutLogger', () => {
       });
     });
 
-    it.each([
+    type SurfaceDetectionTestCase = {
+      name: string;
+      env: Record<string, string | undefined>;
+      expected: string;
+    };
+
+    it.each<SurfaceDetectionTestCase>([
       {
-        env: {
-          CURSOR_TRACE_ID: 'abc123',
-          GITHUB_SHA: undefined,
-          TERM_PROGRAM: 'vscode',
-        },
-        expectedValue: 'cursor',
+        name: 'github action',
+        env: { GITHUB_SHA: '8675309' },
+        expected: 'GitHub',
       },
       {
+        name: 'Cloud Shell via EDITOR_IN_CLOUD_SHELL',
+        env: { EDITOR_IN_CLOUD_SHELL: 'true' },
+        expected: 'cloudshell',
+      },
+      {
+        name: 'Cloud Shell via CLOUD_SHELL',
+        env: { CLOUD_SHELL: 'true' },
+        expected: 'cloudshell',
+      },
+      {
+        name: 'VSCode via TERM_PROGRAM',
         env: {
           TERM_PROGRAM: 'vscode',
           GITHUB_SHA: undefined,
           MONOSPACE_ENV: '',
         },
-        expectedValue: 'vscode',
+        expected: 'vscode',
       },
       {
+        name: 'SURFACE env var',
+        env: { SURFACE: 'ide-1234' },
+        expected: 'ide-1234',
+      },
+      {
+        name: 'SURFACE env var takes precedence',
+        env: { TERM_PROGRAM: 'vscode', SURFACE: 'ide-1234' },
+        expected: 'ide-1234',
+      },
+      {
+        name: 'Cursor',
+        env: {
+          CURSOR_TRACE_ID: 'abc123',
+          TERM_PROGRAM: 'vscode',
+          GITHUB_SHA: undefined,
+        },
+        expected: 'cursor',
+      },
+      {
+        name: 'Firebase Studio',
         env: {
           MONOSPACE_ENV: 'true',
-          GITHUB_SHA: undefined,
           TERM_PROGRAM: 'vscode',
+          GITHUB_SHA: undefined,
         },
-        expectedValue: 'firebasestudio',
+        expected: 'firebasestudio',
       },
       {
+        name: 'Devin',
         env: {
           __COG_BASHRC_SOURCED: 'true',
-          GITHUB_SHA: undefined,
           TERM_PROGRAM: 'vscode',
+          GITHUB_SHA: undefined,
         },
-        expectedValue: 'devin',
+        expected: 'devin',
       },
       {
+        name: 'unidentified',
         env: {
-          CLOUD_SHELL: 'true',
           GITHUB_SHA: undefined,
-          TERM_PROGRAM: 'vscode',
+          TERM_PROGRAM: undefined,
+          SURFACE: undefined,
         },
-        expectedValue: 'cloudshell',
+        expected: 'SURFACE_NOT_SET',
       },
     ])(
-      'logs the current surface as $expectedValue, preempting vscode detection',
-      ({ env, expectedValue }) => {
+      'logs the current surface as $expected from $name',
+      ({ env, expected }) => {
         const { logger } = setup({});
         for (const [key, value] of Object.entries(env)) {
           vi.stubEnv(key, value);
         }
-        // Clear Cursor-specific environment variables that might interfere with tests
-        // Only clear if not explicitly testing Cursor detection
-        if (!env.CURSOR_TRACE_ID) {
-          vi.stubEnv('CURSOR_TRACE_ID', '');
-        }
         const event = logger?.createLogEvent(EventNames.API_ERROR, []);
-        expect(event?.event_metadata[0][3]).toEqual({
+        expect(event?.event_metadata[0]).toContainEqual({
           gemini_cli_key: EventMetadataKey.GEMINI_CLI_SURFACE,
-          value: expectedValue,
+          value: expected,
         });
       },
     );
   });
 
+  describe('GH_WORKFLOW_NAME metadata', () => {
+    it('includes workflow name when GH_WORKFLOW_NAME is set', () => {
+      const { logger } = setup({});
+      vi.stubEnv('GH_WORKFLOW_NAME', 'test-workflow');
+
+      const event = logger?.createLogEvent(EventNames.API_ERROR, []);
+      expect(event?.event_metadata[0]).toContainEqual({
+        gemini_cli_key: EventMetadataKey.GEMINI_CLI_GH_WORKFLOW_NAME,
+        value: 'test-workflow',
+      });
+    });
+
+    it('does not include workflow name when GH_WORKFLOW_NAME is not set', () => {
+      const { logger } = setup({});
+      vi.stubEnv('GH_WORKFLOW_NAME', undefined);
+
+      const event = logger?.createLogEvent(EventNames.API_ERROR, []);
+      const hasWorkflowName = event?.event_metadata[0].some(
+        (item) =>
+          item.gemini_cli_key === EventMetadataKey.GEMINI_CLI_GH_WORKFLOW_NAME,
+      );
+      expect(hasWorkflowName).toBe(false);
+    });
+  });
+
   describe('logChatCompressionEvent', () => {
     it('logs an event with proper fields', () => {
       const { logger } = setup();
@@ -744,6 +750,23 @@ describe('ClearcutLogger', () => {
     });
   });
 
+  describe('logExperiments', () => {
+    it('logs an event with gws_experiment field containing exp ids', () => {
+      const { logger } = setup();
+      const event = new AgentStartEvent('agent-123', 'TestAgent');
+
+      logger?.logAgentStartEvent(event);
+
+      const events = getEvents(logger!);
+      expect(events.length).toBe(1);
+      expect(events[0]).toHaveEventName(EventNames.AGENT_START);
+      expect(events[0]).toHaveMetadataValue([
+        EventMetadataKey.GEMINI_CLI_EXPERIMENT_IDS,
+        '123,456,789',
+      ]);
+    });
+  });
+
   describe('logAgentFinishEvent', () => {
     it('logs an event with proper fields (success)', () => {
       const { logger } = setup();
@@ -882,8 +905,7 @@ describe('ClearcutLogger', () => {
         status: 'success',
       } as SuccessfulToolCall;
 
-      // eslint-disable-next-line @typescript-eslint/no-explicit-any
-      logger?.logToolCallEvent(new ToolCallEvent(completedToolCall as any));
+      logger?.logToolCallEvent(new ToolCallEvent(completedToolCall));
 
       const events = getEvents(logger!);
       expect(events.length).toBe(1);
@@ -928,8 +950,7 @@ describe('ClearcutLogger', () => {
         status: 'success',
       } as SuccessfulToolCall;
 
-      // eslint-disable-next-line @typescript-eslint/no-explicit-any
-      logger?.logToolCallEvent(new ToolCallEvent(completedToolCall as any));
+      logger?.logToolCallEvent(new ToolCallEvent(completedToolCall));
 
       const events = getEvents(logger!);
       expect(events.length).toBe(1);
@@ -940,6 +961,33 @@ describe('ClearcutLogger', () => {
     });
   });
 
+  describe('flushIfNeeded', () => {
+    it('should not flush if the interval has not passed', () => {
+      const { logger } = setup();
+      const flushSpy = vi
+        // eslint-disable-next-line @typescript-eslint/no-explicit-any
+        .spyOn(logger!, 'flushToClearcut' as any)
+        .mockResolvedValue({ nextRequestWaitMs: 0 });
+
+      logger!.flushIfNeeded();
+      expect(flushSpy).not.toHaveBeenCalled();
+    });
+
+    it('should flush if the interval has passed', async () => {
+      const { logger } = setup();
+      const flushSpy = vi
+        // eslint-disable-next-line @typescript-eslint/no-explicit-any
+        .spyOn(logger!, 'flushToClearcut' as any)
+        .mockResolvedValue({ nextRequestWaitMs: 0 });
+
+      // Advance time by more than the flush interval
+      await vi.advanceTimersByTimeAsync(1000 * 60 * 2);
+
+      logger!.flushIfNeeded();
+      expect(flushSpy).toHaveBeenCalled();
+    });
+  });
+
   describe('logWebFetchFallbackAttemptEvent', () => {
     it('logs an event with the proper name and reason', () => {
       const { logger } = setup();
