@@ -18,6 +18,7 @@ import fs from 'node:fs';
 import fsp from 'node:fs/promises';
 import os from 'node:os';
 import * as path from 'node:path';
+import { debugLogger } from '../utils/debugLogger.js';
 
 // Mock dependencies
 const mockOpenDiff = vi.hoisted(() => vi.fn());
@@ -85,18 +86,25 @@ describe('modifyWithEditor', () => {
   afterEach(async () => {
     vi.restoreAllMocks();
     await fsp.rm(testProjectDir, { recursive: true, force: true });
-    const diffDir = path.join(os.tmpdir(), 'gemini-cli-tool-modify-diffs');
-    await fsp.rm(diffDir, { recursive: true, force: true });
   });
 
   describe('successful modification', () => {
+    const assertMode = (mode: number, expected: number): void => {
+      if (process.platform === 'win32') {
+        // Windows reports POSIX modes as 0o666 regardless of requested bits.
+        // At minimum confirm the owner read/write bits are present.
+        expect(mode & 0o600).toBe(0o600);
+        return;
+      }
+      expect(mode & 0o777).toBe(expected);
+    };
+
     it('should successfully modify content with VSCode editor', async () => {
       const result = await modifyWithEditor(
         mockParams,
         mockModifyContext,
         'vscode' as EditorType,
         abortSignal,
-        vi.fn(),
       );
 
       expect(mockModifyContext.getCurrentContent).toHaveBeenCalledWith(
@@ -142,38 +150,28 @@ describe('modifyWithEditor', () => {
       });
     });
 
-    it('should create temp directory if it does not exist', async () => {
-      const diffDir = path.join(os.tmpdir(), 'gemini-cli-tool-modify-diffs');
-      await fsp.rm(diffDir, { recursive: true, force: true }).catch(() => {});
+    it('should create temp directory and files with restrictive permissions', async () => {
+      mockOpenDiff.mockImplementation(async (oldPath, newPath) => {
+        const diffDir = path.dirname(oldPath);
+        expect(diffDir).toBe(path.dirname(newPath));
+
+        const dirStats = await fsp.stat(diffDir);
+        const oldStats = await fsp.stat(oldPath);
+        const newStats = await fsp.stat(newPath);
+
+        assertMode(dirStats.mode, 0o700);
+        assertMode(oldStats.mode, 0o600);
+        assertMode(newStats.mode, 0o600);
+
+        await fsp.writeFile(newPath, modifiedContent, 'utf8');
+      });
 
       await modifyWithEditor(
         mockParams,
         mockModifyContext,
         'vscode' as EditorType,
         abortSignal,
-        vi.fn(),
       );
-
-      const stats = await fsp.stat(diffDir);
-      expect(stats.isDirectory()).toBe(true);
-    });
-
-    it('should not create temp directory if it already exists', async () => {
-      const diffDir = path.join(os.tmpdir(), 'gemini-cli-tool-modify-diffs');
-      await fsp.mkdir(diffDir, { recursive: true });
-
-      const mkdirSpy = vi.spyOn(fs, 'mkdirSync');
-
-      await modifyWithEditor(
-        mockParams,
-        mockModifyContext,
-        'vscode' as EditorType,
-        abortSignal,
-        vi.fn(),
-      );
-
-      expect(mkdirSpy).not.toHaveBeenCalled();
-      mkdirSpy.mockRestore();
     });
   });
 
@@ -188,7 +186,6 @@ describe('modifyWithEditor', () => {
       mockModifyContext,
       'vscode' as EditorType,
       abortSignal,
-      vi.fn(),
     );
 
     expect(mockCreatePatch).toHaveBeenCalledWith(
@@ -217,7 +214,6 @@ describe('modifyWithEditor', () => {
       mockModifyContext,
       'vscode' as EditorType,
       abortSignal,
-      vi.fn(),
     );
 
     expect(mockCreatePatch).toHaveBeenCalledWith(
@@ -236,6 +232,62 @@ describe('modifyWithEditor', () => {
     expect(result.updatedDiff).toBe('mock diff content');
   });
 
+  it('should honor override content values when provided', async () => {
+    const overrideCurrent = 'override current content';
+    const overrideProposed = 'override proposed content';
+    mockModifyContext.getCurrentContent = vi.fn();
+    mockModifyContext.getProposedContent = vi.fn();
+
+    await modifyWithEditor(
+      mockParams,
+      mockModifyContext,
+      'vscode' as EditorType,
+      abortSignal,
+      {
+        currentContent: overrideCurrent,
+        proposedContent: overrideProposed,
+      },
+    );
+
+    expect(mockModifyContext.getCurrentContent).not.toHaveBeenCalled();
+    expect(mockModifyContext.getProposedContent).not.toHaveBeenCalled();
+    expect(mockCreatePatch).toHaveBeenCalledWith(
+      path.basename(mockParams.filePath),
+      overrideCurrent,
+      modifiedContent,
+      'Current',
+      'Proposed',
+      expect.any(Object),
+    );
+  });
+
+  it('should treat null override as explicit empty content', async () => {
+    mockModifyContext.getCurrentContent = vi.fn();
+    mockModifyContext.getProposedContent = vi.fn();
+
+    await modifyWithEditor(
+      mockParams,
+      mockModifyContext,
+      'vscode' as EditorType,
+      abortSignal,
+      {
+        currentContent: null,
+        proposedContent: 'override proposed content',
+      },
+    );
+
+    expect(mockModifyContext.getCurrentContent).not.toHaveBeenCalled();
+    expect(mockModifyContext.getProposedContent).not.toHaveBeenCalled();
+    expect(mockCreatePatch).toHaveBeenCalledWith(
+      path.basename(mockParams.filePath),
+      '',
+      modifiedContent,
+      'Current',
+      'Proposed',
+      expect.any(Object),
+    );
+  });
+
   it('should clean up temp files even if editor fails', async () => {
     const editorError = new Error('Editor failed to open');
     mockOpenDiff.mockRejectedValue(editorError);
@@ -248,7 +300,6 @@ describe('modifyWithEditor', () => {
         mockModifyContext,
         'vscode' as EditorType,
         abortSignal,
-        vi.fn(),
       ),
     ).rejects.toThrow('Editor failed to open');
 
@@ -264,24 +315,29 @@ describe('modifyWithEditor', () => {
 
   it('should handle temp file cleanup errors gracefully', async () => {
     const consoleErrorSpy = vi
-      .spyOn(console, 'error')
+      .spyOn(debugLogger, 'error')
       .mockImplementation(() => {});
     vi.spyOn(fs, 'unlinkSync').mockImplementation(() => {
       throw new Error('Failed to delete file');
     });
+    vi.spyOn(fs, 'rmdirSync').mockImplementation(() => {
+      throw new Error('Failed to delete directory');
+    });
 
     await modifyWithEditor(
       mockParams,
       mockModifyContext,
       'vscode' as EditorType,
       abortSignal,
-      vi.fn(),
     );
 
-    expect(consoleErrorSpy).toHaveBeenCalledTimes(2);
+    expect(consoleErrorSpy).toHaveBeenCalledTimes(3);
     expect(consoleErrorSpy).toHaveBeenCalledWith(
       expect.stringContaining('Error deleting temp diff file:'),
     );
+    expect(consoleErrorSpy).toHaveBeenCalledWith(
+      expect.stringContaining('Error deleting temp diff directory:'),
+    );
 
     consoleErrorSpy.mockRestore();
   });
@@ -299,7 +355,6 @@ describe('modifyWithEditor', () => {
       mockModifyContext,
       'vscode' as EditorType,
       abortSignal,
-      vi.fn(),
     );
 
     expect(mockOpenDiff).toHaveBeenCalledOnce();
@@ -307,9 +362,9 @@ describe('modifyWithEditor', () => {
     expect(oldFilePath).toMatch(/gemini-cli-modify-test-file-old-\d+\.txt$/);
     expect(newFilePath).toMatch(/gemini-cli-modify-test-file-new-\d+\.txt$/);
 
-    const diffDir = path.join(os.tmpdir(), 'gemini-cli-tool-modify-diffs');
-    expect(path.dirname(oldFilePath)).toBe(diffDir);
-    expect(path.dirname(newFilePath)).toBe(diffDir);
+    const diffDirPrefix = path.join(os.tmpdir(), 'gemini-cli-tool-modify-');
+    expect(path.dirname(oldFilePath).startsWith(diffDirPrefix)).toBe(true);
+    expect(path.dirname(newFilePath).startsWith(diffDirPrefix)).toBe(true);
   });
 
   it('should create temp files with correct naming without extension', async () => {
@@ -321,7 +376,6 @@ describe('modifyWithEditor', () => {
       mockModifyContext,
       'vscode' as EditorType,
       abortSignal,
-      vi.fn(),
     );
 
     expect(mockOpenDiff).toHaveBeenCalledOnce();
@@ -329,9 +383,9 @@ describe('modifyWithEditor', () => {
     expect(oldFilePath).toMatch(/gemini-cli-modify-test-file-old-\d+$/);
     expect(newFilePath).toMatch(/gemini-cli-modify-test-file-new-\d+$/);
 
-    const diffDir = path.join(os.tmpdir(), 'gemini-cli-tool-modify-diffs');
-    expect(path.dirname(oldFilePath)).toBe(diffDir);
-    expect(path.dirname(newFilePath)).toBe(diffDir);
+    const diffDirPrefix = path.join(os.tmpdir(), 'gemini-cli-tool-modify-');
+    expect(path.dirname(oldFilePath).startsWith(diffDirPrefix)).toBe(true);
+    expect(path.dirname(newFilePath).startsWith(diffDirPrefix)).toBe(true);
   });
 });
 
