@@ -88,6 +88,7 @@ describe('ReadManyFilesTool', () => {
         buildExcludePatterns: () => DEFAULT_FILE_EXCLUDES,
         getReadManyFilesExcludes: () => DEFAULT_FILE_EXCLUDES,
       }),
+      isInteractive: () => false,
     } as Partial<Config> as Config;
     tool = new ReadManyFilesTool(mockConfig);
 
@@ -144,56 +145,56 @@ describe('ReadManyFilesTool', () => {
 
   describe('build', () => {
     it('should return an invocation for valid relative paths within root', () => {
-      const params = { paths: ['file1.txt', 'subdir/file2.txt'] };
+      const params = { include: ['file1.txt', 'subdir/file2.txt'] };
       const invocation = tool.build(params);
       expect(invocation).toBeDefined();
     });
 
     it('should return an invocation for valid glob patterns within root', () => {
-      const params = { paths: ['*.txt', 'subdir/**/*.js'] };
+      const params = { include: ['*.txt', 'subdir/**/*.js'] };
       const invocation = tool.build(params);
       expect(invocation).toBeDefined();
     });
 
     it('should return an invocation for paths trying to escape the root (e.g., ../) as execute handles this', () => {
-      const params = { paths: ['../outside.txt'] };
+      const params = { include: ['../outside.txt'] };
       const invocation = tool.build(params);
       expect(invocation).toBeDefined();
     });
 
     it('should return an invocation for absolute paths as execute handles this', () => {
-      const params = { paths: [path.join(tempDirOutsideRoot, 'absolute.txt')] };
+      const params = {
+        include: [path.join(tempDirOutsideRoot, 'absolute.txt')],
+      };
       const invocation = tool.build(params);
       expect(invocation).toBeDefined();
     });
 
     it('should throw error if paths array is empty', () => {
-      const params = { paths: [] };
+      const params = { include: [] };
       expect(() => tool.build(params)).toThrow(
-        'params/paths must NOT have fewer than 1 items',
+        'params/include must NOT have fewer than 1 items',
       );
     });
 
     it('should return an invocation for valid exclude and include patterns', () => {
       const params = {
-        paths: ['src/**/*.ts'],
         exclude: ['**/*.test.ts'],
-        include: ['src/utils/*.ts'],
+        include: ['src/**/*.ts', 'src/utils/*.ts'],
       };
       const invocation = tool.build(params);
       expect(invocation).toBeDefined();
     });
 
     it('should throw error if paths array contains an empty string', () => {
-      const params = { paths: ['file1.txt', ''] };
+      const params = { include: ['file1.txt', ''] };
       expect(() => tool.build(params)).toThrow(
-        'params/paths/1 must NOT have fewer than 1 characters',
+        'params/include/1 must NOT have fewer than 1 characters',
       );
     });
 
     it('should throw error if include array contains non-string elements', () => {
       const params = {
-        paths: ['file1.txt'],
         include: ['*.ts', 123] as string[],
       };
       expect(() => tool.build(params)).toThrow(
@@ -203,7 +204,7 @@ describe('ReadManyFilesTool', () => {
 
     it('should throw error if exclude array contains non-string elements', () => {
       const params = {
-        paths: ['file1.txt'],
+        include: ['file1.txt'],
         exclude: ['*.log', {}] as string[],
       };
       expect(() => tool.build(params)).toThrow(
@@ -226,7 +227,7 @@ describe('ReadManyFilesTool', () => {
 
     it('should read a single specified file', async () => {
       createFile('file1.txt', 'Content of file1');
-      const params = { paths: ['file1.txt'] };
+      const params = { include: ['file1.txt'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const expectedPath = path.join(tempRootDir, 'file1.txt');
@@ -242,7 +243,7 @@ describe('ReadManyFilesTool', () => {
     it('should read multiple specified files', async () => {
       createFile('file1.txt', 'Content1');
       createFile('subdir/file2.js', 'Content2');
-      const params = { paths: ['file1.txt', 'subdir/file2.js'] };
+      const params = { include: ['file1.txt', 'subdir/file2.js'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const content = result.llmContent as string[];
@@ -267,7 +268,7 @@ describe('ReadManyFilesTool', () => {
       createFile('file.txt', 'Text file');
       createFile('another.txt', 'Another text');
       createFile('sub/data.json', '{}');
-      const params = { paths: ['*.txt'] };
+      const params = { include: ['*.txt'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const content = result.llmContent as string[];
@@ -292,7 +293,7 @@ describe('ReadManyFilesTool', () => {
     it('should respect exclude patterns', async () => {
       createFile('src/main.ts', 'Main content');
       createFile('src/main.test.ts', 'Test content');
-      const params = { paths: ['src/**/*.ts'], exclude: ['**/*.test.ts'] };
+      const params = { include: ['src/**/*.ts'], exclude: ['**/*.test.ts'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const content = result.llmContent as string[];
@@ -310,7 +311,7 @@ describe('ReadManyFilesTool', () => {
     });
 
     it('should handle nonexistent specific files gracefully', async () => {
-      const params = { paths: ['nonexistent-file.txt'] };
+      const params = { include: ['nonexistent-file.txt'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       expect(result.llmContent).toEqual([
@@ -324,7 +325,7 @@ describe('ReadManyFilesTool', () => {
     it('should use default excludes', async () => {
       createFile('node_modules/some-lib/index.js', 'lib code');
       createFile('src/app.js', 'app code');
-      const params = { paths: ['**/*.js'] };
+      const params = { include: ['**/*.js'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const content = result.llmContent as string[];
@@ -344,7 +345,7 @@ describe('ReadManyFilesTool', () => {
     it('should NOT use default excludes if useDefaultExcludes is false', async () => {
       createFile('node_modules/some-lib/index.js', 'lib code');
       createFile('src/app.js', 'app code');
-      const params = { paths: ['**/*.js'], useDefaultExcludes: false };
+      const params = { include: ['**/*.js'], useDefaultExcludes: false };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const content = result.llmContent as string[];
@@ -373,7 +374,7 @@ describe('ReadManyFilesTool', () => {
         'image.png',
         Buffer.from([0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a]),
       );
-      const params = { paths: ['*.png'] }; // Explicitly requesting .png
+      const params = { include: ['*.png'] }; // Explicitly requesting .png
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       expect(result.llmContent).toEqual([
@@ -397,7 +398,7 @@ describe('ReadManyFilesTool', () => {
         'myExactImage.png',
         Buffer.from([0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a]),
       );
-      const params = { paths: ['myExactImage.png'] }; // Explicitly requesting by full name
+      const params = { include: ['myExactImage.png'] }; // Explicitly requesting by full name
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       expect(result.llmContent).toEqual([
@@ -416,7 +417,7 @@ describe('ReadManyFilesTool', () => {
     it('should skip PDF files if not explicitly requested by extension or name', async () => {
       createBinaryFile('document.pdf', Buffer.from('%PDF-1.4...'));
       createFile('notes.txt', 'text notes');
-      const params = { paths: ['*'] }; // Generic glob, not specific to .pdf
+      const params = { include: ['*'] }; // Generic glob, not specific to .pdf
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const content = result.llmContent as string[];
@@ -436,7 +437,7 @@ describe('ReadManyFilesTool', () => {
 
     it('should include PDF files as inlineData parts if explicitly requested by extension', async () => {
       createBinaryFile('important.pdf', Buffer.from('%PDF-1.4...'));
-      const params = { paths: ['*.pdf'] }; // Explicitly requesting .pdf files
+      const params = { include: ['*.pdf'] }; // Explicitly requesting .pdf files
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       expect(result.llmContent).toEqual([
@@ -452,7 +453,7 @@ describe('ReadManyFilesTool', () => {
 
     it('should include PDF files as inlineData parts if explicitly requested by name', async () => {
       createBinaryFile('report-final.pdf', Buffer.from('%PDF-1.4...'));
-      const params = { paths: ['report-final.pdf'] };
+      const params = { include: ['report-final.pdf'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       expect(result.llmContent).toEqual([
@@ -470,7 +471,7 @@ describe('ReadManyFilesTool', () => {
       createFile('foo.bar', '');
       createFile('bar.ts', '');
       createFile('foo.quux', '');
-      const params = { paths: ['foo.bar', 'bar.ts', 'foo.quux'] };
+      const params = { include: ['foo.bar', 'bar.ts', 'foo.quux'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       expect(result.returnDisplay).not.toContain('foo.bar');
@@ -502,13 +503,14 @@ describe('ReadManyFilesTool', () => {
           buildExcludePatterns: () => [],
           getReadManyFilesExcludes: () => [],
         }),
+        isInteractive: () => false,
       } as Partial<Config> as Config;
       tool = new ReadManyFilesTool(mockConfig);
 
       fs.writeFileSync(path.join(tempDir1, 'file1.txt'), 'Content1');
       fs.writeFileSync(path.join(tempDir2, 'file2.txt'), 'Content2');
 
-      const params = { paths: ['*.txt'] };
+      const params = { include: ['*.txt'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const content = result.llmContent as string[];
@@ -544,7 +546,7 @@ describe('ReadManyFilesTool', () => {
       );
       createFile('large-file.txt', longContent);
 
-      const params = { paths: ['*.txt'] };
+      const params = { include: ['*.txt'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const content = result.llmContent as string[];
@@ -568,7 +570,7 @@ describe('ReadManyFilesTool', () => {
     it('should read files with special characters like [] and () in the path', async () => {
       const filePath = 'src/app/[test]/(dashboard)/testing/components/code.tsx';
       createFile(filePath, 'Content of receive-detail');
-      const params = { paths: [filePath] };
+      const params = { include: [filePath] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const expectedPath = path.join(tempRootDir, filePath);
@@ -587,7 +589,7 @@ Content of receive-detail
 
     it('should read files with special characters in the name', async () => {
       createFile('file[1].txt', 'Content of file[1]');
-      const params = { paths: ['file[1].txt'] };
+      const params = { include: ['file[1].txt'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       const expectedPath = path.join(tempRootDir, 'file[1].txt');
@@ -607,15 +609,15 @@ Content of file[1]
 
   describe('Error handling', () => {
     it('should return an INVALID_TOOL_PARAMS error if no paths are provided', async () => {
-      const params = { paths: [], include: [] };
+      const params = { include: [] };
       expect(() => {
         tool.build(params);
-      }).toThrow('params/paths must NOT have fewer than 1 items');
+      }).toThrow('params/include must NOT have fewer than 1 items');
     });
 
     it('should return a READ_MANY_FILES_SEARCH_ERROR on glob failure', async () => {
       vi.mocked(glob.glob).mockRejectedValue(new Error('Glob failed'));
-      const params = { paths: ['*.txt'] };
+      const params = { include: ['*.txt'] };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
       expect(result.error?.type).toBe(
@@ -661,7 +663,7 @@ Content of file[1]
         return 'text';
       });
 
-      const params = { paths: files };
+      const params = { include: files };
       const invocation = tool.build(params);
       const result = await invocation.execute(new AbortController().signal);
 
@@ -684,7 +686,7 @@ Content of file[1]
       createFile('valid3.txt', 'Valid content 3');
 
       const params = {
-        paths: [
+        include: [
           'valid1.txt',
           'valid2.txt',
           'nonexistent-file.txt', // This will fail
@@ -730,7 +732,7 @@ Content of file[1]
         return 'text';
       });
 
-      const invocation = tool.build({ paths: files });
+      const invocation = tool.build({ include: files });
       await invocation.execute(new AbortController().signal);
 
       // Verify concurrent execution pattern
