@@ -35,18 +35,10 @@ import { READ_MANY_FILES_TOOL_NAME } from './tool-names.js';
  */
 export interface ReadManyFilesParams {
   /**
-   * An array of file paths or directory paths to search within.
-   * Paths are relative to the tool's configured target directory.
-   * Glob patterns can be used directly in these paths.
-   */
-  paths: string[];
-
-  /**
-   * Optional. Glob patterns for files to include.
-   * These are effectively combined with the `paths`.
+   * Glob patterns for files to include.
    * Example: ["*.ts", "src/** /*.md"]
    */
-  include?: string[];
+  include: string[];
 
   /**
    * Optional. Glob patterns for files/directories to exclude.
@@ -123,9 +115,8 @@ class ReadManyFilesToolInvocation extends BaseToolInvocation<
   }
 
   getDescription(): string {
-    const allPatterns = [...this.params.paths, ...(this.params.include || [])];
     const pathDesc = `using patterns: 
-${allPatterns.join('`, `')}
+${this.params.include.join('`, `')}
  (within target directory: 
 ${this.config.getTargetDir()}
 ) `;
@@ -133,19 +124,12 @@ ${this.config.getTargetDir()}
     // Determine the final list of exclusion patterns exactly as in execute method
     const paramExcludes = this.params.exclude || [];
     const paramUseDefaultExcludes = this.params.useDefaultExcludes !== false;
-    const geminiIgnorePatterns = this.config
-      .getFileService()
-      .getGeminiIgnorePatterns();
     const finalExclusionPatternsForDescription: string[] =
       paramUseDefaultExcludes
-        ? [
-            ...getDefaultExcludes(this.config),
-            ...paramExcludes,
-            ...geminiIgnorePatterns,
-          ]
-        : [...paramExcludes, ...geminiIgnorePatterns];
+        ? [...getDefaultExcludes(this.config), ...paramExcludes]
+        : [...paramExcludes];
 
-    let excludeDesc = `Excluding: ${
+    const excludeDesc = `Excluding: ${
       finalExclusionPatternsForDescription.length > 0
         ? `patterns like 
 ${finalExclusionPatternsForDescription
@@ -156,16 +140,6 @@ ${finalExclusionPatternsForDescription
         : 'none specified'
     }`;
 
-    // Add a note if .geminiignore patterns contributed to the final list of exclusions
-    if (geminiIgnorePatterns.length > 0) {
-      const geminiPatternsInEffect = geminiIgnorePatterns.filter((p) =>
-        finalExclusionPatternsForDescription.includes(p),
-      ).length;
-      if (geminiPatternsInEffect > 0) {
-        excludeDesc += ` (includes ${geminiPatternsInEffect} from .geminiignore)`;
-      }
-    }
-
     return `Will attempt to read and concatenate files ${pathDesc}. ${excludeDesc}. File encoding: ${DEFAULT_ENCODING}. Separator: "${DEFAULT_OUTPUT_SEPARATOR_FORMAT.replace(
       '{filePath}',
       'path/to/file.ext',
@@ -173,12 +147,7 @@ ${finalExclusionPatternsForDescription
   }
 
   async execute(signal: AbortSignal): Promise<ToolResult> {
-    const {
-      paths: inputPatterns,
-      include = [],
-      exclude = [],
-      useDefaultExcludes = true,
-    } = this.params;
+    const { include, exclude = [], useDefaultExcludes = true } = this.params;
 
     const filesToConsider = new Set<string>();
     const skippedFiles: Array<{ path: string; reason: string }> = [];
@@ -189,14 +158,13 @@ ${finalExclusionPatternsForDescription
       ? [...getDefaultExcludes(this.config), ...exclude]
       : [...exclude];
 
-    const searchPatterns = [...inputPatterns, ...include];
     try {
       const allEntries = new Set<string>();
       const workspaceDirs = this.config.getWorkspaceContext().getDirectories();
 
       for (const dir of workspaceDirs) {
         const processedPatterns = [];
-        for (const p of searchPatterns) {
+        for (const p of include) {
           const normalizedP = p.replace(/\\/g, '/');
           const fullPath = path.join(dir, normalizedP);
           if (fs.existsSync(fullPath)) {
@@ -225,7 +193,7 @@ ${finalExclusionPatternsForDescription
       );
 
       const fileDiscovery = this.config.getFileService();
-      const { filteredPaths, gitIgnoredCount, geminiIgnoredCount } =
+      const { filteredPaths, ignoredCount } =
         fileDiscovery.filterFilesWithReport(relativeEntries, {
           respectGitIgnore:
             this.params.file_filtering_options?.respect_git_ignore ??
@@ -253,19 +221,11 @@ ${finalExclusionPatternsForDescription
         filesToConsider.add(fullPath);
       }
 
-      // Add info about git-ignored files if any were filtered
-      if (gitIgnoredCount > 0) {
+      // Add info about ignored files if any were filtered
+      if (ignoredCount > 0) {
         skippedFiles.push({
-          path: `${gitIgnoredCount} file(s)`,
-          reason: 'git ignored',
-        });
-      }
-
-      // Add info about gemini-ignored files if any were filtered
-      if (geminiIgnoredCount > 0) {
-        skippedFiles.push({
-          path: `${geminiIgnoredCount} file(s)`,
-          reason: 'gemini ignored',
+          path: `${ignoredCount} file(s)`,
+          reason: 'ignored by project ignore files',
         });
       }
     } catch (error) {
@@ -297,7 +257,7 @@ ${finalExclusionPatternsForDescription
               filePath,
               fileExtension,
             );
-            const requestedExplicitly = inputPatterns.some(
+            const requestedExplicitly = include.some(
               (pattern: string) =>
                 pattern.toLowerCase().includes(fileExtension) ||
                 pattern.includes(fileNameWithoutExtension),
@@ -392,7 +352,7 @@ ${finalExclusionPatternsForDescription
               : undefined;
           const mimetype = getSpecificMimeType(filePath);
           const programming_language = getProgrammingLanguage({
-            absolute_path: filePath,
+            file_path: filePath,
           });
           logFileOperation(
             this.config,
@@ -488,7 +448,7 @@ export class ReadManyFilesTool extends BaseDeclarativeTool<
     const parameterSchema = {
       type: 'object',
       properties: {
-        paths: {
+        include: {
           type: 'array',
           items: {
             type: 'string',
@@ -496,17 +456,7 @@ export class ReadManyFilesTool extends BaseDeclarativeTool<
           },
           minItems: 1,
           description:
-            "Required. An array of glob patterns or paths relative to the tool's target directory. Examples: ['src/**/*.ts'], ['README.md', 'docs/']",
-        },
-        include: {
-          type: 'array',
-          items: {
-            type: 'string',
-            minLength: 1,
-          },
-          description:
-            'Optional. Additional glob patterns to include. These are merged with `paths`. Example: "*.test.ts" to specifically add test files if they were broadly excluded.',
-          default: [],
+            'An array of glob patterns or paths. Examples: ["src/**/*.ts"], ["README.md", "docs/"]',
         },
         exclude: {
           type: 'array',
@@ -548,13 +498,13 @@ export class ReadManyFilesTool extends BaseDeclarativeTool<
           },
         },
       },
-      required: ['paths'],
+      required: ['include'],
     };
 
     super(
       ReadManyFilesTool.Name,
       'ReadManyFiles',
-      `Reads content from multiple files specified by paths or glob patterns within a configured target directory. For text files, it concatenates their content into a single string. It is primarily designed for text-based files. However, it can also process image (e.g., .png, .jpg) and PDF (.pdf) files if their file names or extensions are explicitly included in the 'paths' argument. For these explicitly requested non-text files, their data is read and included in a format suitable for model consumption (e.g., base64 encoded).
+      `Reads content from multiple files specified by glob patterns within a configured target directory. For text files, it concatenates their content into a single string. It is primarily designed for text-based files. However, it can also process image (e.g., .png, .jpg) and PDF (.pdf) files if their file names or extensions are explicitly included in the 'include' argument. For these explicitly requested non-text files, their data is read and included in a format suitable for model consumption (e.g., base64 encoded).
 
 This tool is useful when you need to understand or analyze a collection of files, such as:
 - Getting an overview of a codebase or parts of it (e.g., all TypeScript files in the 'src' directory).
@@ -563,7 +513,7 @@ This tool is useful when you need to understand or analyze a collection of files
 - Gathering context from multiple configuration files.
 - When the user asks to "read all files in X directory" or "show me the content of all Y files".
 
-Use this tool when the user's query implies needing the content of several files simultaneously for context, analysis, or summarization. For text files, it uses default UTF-8 encoding and a '--- {filePath} ---' separator between file contents. The tool inserts a '--- End of content ---' after the last file. Ensure paths are relative to the target directory. Glob patterns like 'src/**/*.js' are supported. Avoid using for single files if a more specific single-file reading tool is available, unless the user specifically requests to process a list containing just one file via this tool. Other binary files (not explicitly requested as image/PDF) are generally skipped. Default excludes apply to common non-text files (except for explicitly requested images/PDFs) and large dependency directories unless 'useDefaultExcludes' is false.`,
+Use this tool when the user's query implies needing the content of several files simultaneously for context, analysis, or summarization. For text files, it uses default UTF-8 encoding and a '--- {filePath} ---' separator between file contents. The tool inserts a '--- End of content ---' after the last file. Ensure glob patterns are relative to the target directory. Glob patterns like 'src/**/*.js' are supported. Avoid using for single files if a more specific single-file reading tool is available, unless the user specifically requests to process a list containing just one file via this tool. Other binary files (not explicitly requested as image/PDF) are generally skipped. Default excludes apply to common non-text files (except for explicitly requested images/PDFs) and large dependency directories unless 'useDefaultExcludes' is false.`,
       Kind.Read,
       parameterSchema,
       true, // isOutputMarkdown
